/**
 * erwin - really simple html editor
 * Copyright (C) 2004 Adrian Reber
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created on: Mon May 31 01:08:46 2004
 *
 * $Id: configdialog.cc,v 1.9 2004/04/21 20:47:25 adrian Exp $
 *
 * $Author: adrian $
 * 
 * $Log: configdialog.cc,v $
 */

#include <string.h>
#include <stdlib.h>
#include "configdialog.h"
#include "erwindef.h"

extern GtkWidget *main_window;

GtkWidget *highlighting_languages()
{
	GtkListStore *store;
	GtkWidget *combo_box;
	struct document *document;
	GSList *languages = NULL;
	gpointer blub;
	GtkTreeIter iter, preferred;
	int i = 0;
	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();

	memset(&preferred, 0, sizeof(preferred));
	store = gtk_list_store_new(2, G_TYPE_POINTER, G_TYPE_STRING);
	document =
	    (g_list_nth(documents,
		gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook_main))))->data;
	languages =
	    (GSList *) gtk_source_languages_manager_get_available_languages(document->manager);
	while ((blub = g_slist_nth_data(languages, i++))) {
		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter, 0, blub, 1, gtk_source_language_get_name(blub),
		    -1);
		if (globals.default_highlighting
		    && !strcmp(globals.default_highlighting, gtk_source_language_get_name(blub)))
			preferred = iter;
	}

	combo_box = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_box), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_box), renderer, "text", 1, NULL);
	if (preferred.stamp)
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_box), &preferred);
	return combo_box;
}

void settings()
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkWidget *dialog = gtk_dialog_new_with_buttons("My dialog",
	    GTK_WINDOW(main_window),
	    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_STOCK_CANCEL,
	    GTK_RESPONSE_REJECT,
	    GTK_STOCK_OK,
	    GTK_RESPONSE_ACCEPT,
	    NULL);
	gint result;

	GtkWidget *table = gtk_table_new(5, 5, FALSE);
	GtkWidget *label = gtk_label_new("Number of files in recent menu:");
	GtkWidget *recent_files = gtk_spin_button_new_with_range(1, 10, 1);
	GtkWidget *display_statistics =
	    gtk_check_button_new_with_label("Display document statistic in statusbar");
	GtkWidget *show_quickstart =
	    gtk_check_button_new_with_label("Show quickstart dialog when creating a new file");
	GtkWidget *default_language = gtk_label_new("Default highlighting mode:");
	GtkWidget *combo_box = highlighting_languages();

	gtk_table_set_row_spacings(GTK_TABLE(table), 3);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), recent_files, 1, 2, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), display_statistics, 0, 2, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), default_language, 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(table), combo_box, 1, 2, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(table), show_quickstart, 0, 2, 3, 4);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(display_statistics),(globals.display_statistic) ? TRUE : FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_quickstart),(globals.show_quickstart) ? TRUE : FALSE);
	gtk_widget_show_all(GTK_WIDGET(dialog));

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(recent_files), globals.number_of_recent);

	result = gtk_dialog_run(GTK_DIALOG(dialog));
	if (result == GTK_RESPONSE_REJECT || result == GTK_RESPONSE_DELETE_EVENT)
		goto settings_end;

	globals.number_of_recent = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(recent_files));
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo_box), &iter)) {
		if (globals.default_highlighting)
			free(globals.default_highlighting);
		globals.default_highlighting = NULL;
		model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo_box));
		gtk_tree_model_get(model, &iter, 1, &globals.default_highlighting, -1);
	}
	globals.display_statistic = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(display_statistics))) ? 1 : 0;
	globals.show_quickstart = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(show_quickstart))) ? 1 : 0;

      settings_end:
	gtk_widget_destroy(dialog);
}
