/* erwin -- html editor
 * Copyright 1999 David Vogler <david@rhlx01.rz.fht-esslingen.de>
 *                Adrian Reber <adrian@rhlx01.rz.fht-esslingen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef ERWIN_DEF_H
#define ERWIN_DEF_H

#include <gtk/gtk.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>

#define FILENAME "blaaaaaaaaa"

typedef enum {
	LEFT,
	RIGHT
} ErwinCursor;

struct change {
	gint start;
	gint end;
};

struct table {
	gint rows;
	gint cols;
} table_values;

struct frame_entry {
	GtkWidget *entry;
	GtkWidget *frame;
};

struct find_options {
	GtkWidget *case_sensitive;
	GtkWidget *from_beginning;
	GtkWidget *check_button3;
} find_options;

struct body_elements {
	GtkWidget *dialog;
	struct frame_entry title;
	struct frame_entry bgcolor;
	struct frame_entry text;
	struct frame_entry link;
	struct frame_entry vlink;
	struct frame_entry alink;
	struct frame_entry background;
	GtkWidget *insert_generator;
	GtkWidget *content_type;
} body_elements;

struct font_options {
	GtkWidget *dialog;
	GtkWidget *size;
	struct frame_entry face;
	struct frame_entry color;
};

struct frame_combo {
	GtkWidget *entry;
	GtkWidget *frame;
	GtkWidget *combo;
};

struct hyperlink {
	struct frame_entry link;
	struct frame_combo protocol;
	struct frame_entry name;

} hyperlink;

struct document {
	GtkWidget *text;
	GtkWidget *text_view;
	gint nr_of_notebook_child;
	GtkWidget *notebook;
	GtkWidget *notebook_child;
	GtkWidget *notebook_label;
	GString *filename;
	gboolean changed;
	gboolean saved;
	gboolean close_file;
	gboolean is_rendered;
	GtkSourceLanguagesManager *manager;
	GtkSourceLanguage *language;
	GtkWidget *syntax_highlighting_menuitem;
	gboolean syntax_highlighting;
	gboolean new_document;
};

struct global {
	GtkWidget *main_window;
	int number_of_recent;
	gint close_notebook;
	char file[1024];
	gint from_beginning;
	struct font_options font_options;
	GtkWidget *status_bar;
	GtkClipboard *clipboard;
	gchar *default_highlighting;
	gchar *license;
	gint display_statistic;
	gint show_quickstart;
};

extern struct global globals;

gchar *to_be_searched;
GtkWidget *notebook_main;
GList *documents;
gint nr_of_documents;
GtkWidget *notebook_html;
gboolean QUIT;
GtkObject *main_factory;
GSList *highlighting_group;
#endif				/* ERWIN_DEF_H */
