/**
 * erwin - really simple html editor
 * Copyright (C) 1999-2003 Adrian Reber
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: erwinmenu.c,v 1.15 2004/07/03 20:14:25 erwin Exp $
 *
 * $Author: erwin $
 *
 * Description: creating the menus
 *
 * $Log: erwinmenu.c,v $
 * Revision 1.15  2004/07/03 20:14:25  erwin
 * commit
 *
 * Revision 1.14  2004/06/21 21:45:19  erwin
 * big commit
 * it has been a long time without a checkout
 * we are close to version 0.7
 *
 * Revision 1.13  2004/03/11 21:05:01  erwin
 * too many changes without commits
 *
 * Revision 1.12  2003/10/30 20:54:06  erwin
 * syntax highlighting implementation
 *
 * Revision 1.11  2003/10/24 13:31:44  erwin
 * more syntax higlighting related stuff
 *
 */

#ifndef lint
static const char vcid[] = "$Id: erwinmenu.c,v 1.15 2004/07/03 20:14:25 erwin Exp $";
#endif                          /* lint */

#include "erwinfunctions.h"
#include "erwintags.h"
#include "erwindialogs.h"
#include "configdialog.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include "xpm/inline_logo.h"
#include <erwinhighlighting.h>

extern GtkWidget *main_window;

void save_file();

GtkWidget *main_menu(GtkWidget ** menubar, GtkWidget ** recent_menu)
{
        GtkAccelGroup *accel_group = NULL;
        static GtkItemFactoryEntry menu_items[] = {
                {"/_File", NULL, 0, 0, "<Branch>"},
                {"/File/_New", "<control>N", new_file, 0, "<StockItem>",
                    GTK_STOCK_NEW},
                {"/File/_Open", "<control>O", open_file_dlg, 0, "<StockItem>",
                    GTK_STOCK_OPEN},
                {"/File/_Save", "<control>S", save_file, 0, "<StockItem>",
                    GTK_STOCK_SAVE},
                {"/File/Save _As", NULL, save_file_dlg, 0, "<StockItem>",
                    GTK_STOCK_SAVE_AS},
                {"/File/_Recent Files", NULL, 0, 0, "<Branch>"},
                {"/File/_Close", NULL, close_current_file, 0, "<StockItem>", GTK_STOCK_CLOSE},
                {"/File/_Quit", "<control>Q", quit_program_from_menu, 0, "<StockItem>",
                    GTK_STOCK_QUIT},
                {"/_Edit", NULL, 0, 0, "<Branch>"},
                {"/Edit/Cut", "<control>X", cut_to_clipboard, 0, "<StockItem>",
                    GTK_STOCK_CUT},
                {"/Edit/Copy", "<control>C", copy_to_clipboard, 0, "<StockItem>",
                    GTK_STOCK_COPY},
                {"/Edit/Paste", "<control>V", paste_from_clipboard, 0, "<StockItem>",
                    GTK_STOCK_PASTE},
                {"/Edit/sep", NULL, 0, 0, "<Separator>"},
                {"/Edit/Find", "<control>F", find_dlg, 0, "<StockItem>", GTK_STOCK_FIND},
                {"/H_tml", NULL, 0, 0, "<Branch>"},
                {"/Html/Fonts", NULL, 0, 0, "<Branch>"},
                {"/Html/Fonts/Underline <u>", NULL, underline_functions, 0},
                {"/Html/Fonts/Italic <i>", NULL, italic_functions, 0},
                {"/Html/Fonts/Bold <b>", NULL, bold_functions, 0},
                {"/Html/Fonts/H1 <h1>", NULL, h1_functions, 0},
                {"/Html/Fonts/H1 <h2>", NULL, h2_functions, 0},
                {"/Html/Fonts/H1 <h3>", NULL, h3_functions, 0},
                {"/Html/Fonts/Center <center>", NULL, center_functions, 0},
                {"/Html/Fonts/Font Options <font>", NULL, font_dialog, 0},
                {"/Html/Table and Lists", NULL, 0, 0, "<Branch>"},
                {"/Html/Table and Lists/Table <table>", NULL, table_functions, 0},
                {"/Html/Table and Lists/Table Header <th>", NULL, th_functions, 0},
                {"/Html/Table and Lists/Table Row <tr>", NULL, tr_functions, 0},
                {"/Html/Table and Lists/Table Data <td>", NULL, td_functions, 0},
                {"/Html/Table and Lists/Ordered List <ol>", NULL, ol_functions, 0},
                {"/Html/Table and Lists/Unordered List <ul>", NULL, ul_functions, 0},
                {"/Html/Table and Lists/List Item <li>", NULL, li_functions, 0},
                {"/Html/HTML/Insert Link", "<control>L", href_dialog, 0},
                {"/Html/HTML/Insert Image Link", "<control>I", img_functions, 0},
                {"/Html/HTML/Quick Start", NULL, body_functions, 0},
                {"/Html/HTML/Linebreak <br>", "<control>R", br_functions, 0},
                {"/Html/HTML/Paragraph <p>", NULL, p_functions, 0},
                {"/Html/HTML/Space &nbsp;", NULL, nbsp_functions, 0},
                {"/Html/HTML/Emphasis <em>", NULL, em_functions, 0},
                {"/Html/HTML/Strong <strong>", NULL, strong_functions, 0},
                {"/Html/HTML/Small <small>", NULL, small_functions, 0},
                {"/Html/HTML/Strikeout <strike>", NULL, nbsp_functions, 0},
                {"/Html/HTML/Subscript <sub>", NULL, sub_functions, 0},
                {"/Html/HTML/Superscript <sup>", NULL, sup_functions, 0},
                {"/Html/Table Wizard", "<control>T", table_wizard, 0},
                {"/Html/Quickstart", NULL, body_dialog, 0},
                {"/Html/Close Tag", "<control>P", close_tag, 0},
                {"/_Options", NULL, 0, 0, "<Branch>"},
                {"/Options/Syntax Highlighting", NULL, 0, 0, "<Branch>"},
                {"/Options/Syntax Highlighting/Disable", NULL, toggle_syntax_highlighting, 0,
                            "<CheckItem>"},
                {"/Options/Syntax Highlighting/sep", NULL, 0, 0, "<Separator>"},
                {"/_Options/sep", NULL, 0, 0, "<Separator>"},
                {"/_Options/Settings", NULL, settings, 0, 0},
                {"/_Help", NULL, 0, 0, "<LastBranch>"},
                {"/Help/_Legal & License", NULL, legal, 0, "<StockItem>",
                    GTK_STOCK_DIALOG_INFO},
                {"/Help/sep", NULL, 0, 0, "<Separator>"},
                {"/Help/_About", NULL, help_about_menu, 0, "<ImageItem>", inline_logo}
        };
        gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

        accel_group = gtk_accel_group_new();

        main_factory = GTK_OBJECT(gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group));
        gtk_item_factory_create_items(GTK_ITEM_FACTORY(main_factory), nmenu_items, menu_items, NULL);
        gtk_window_add_accel_group(GTK_WINDOW(main_window), accel_group);
        *recent_menu = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(main_factory), "<main>/File/Recent Files");

        if (menubar)
                *menubar = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(main_factory), "<main>");
        return *menubar;
}
